// Copyright (c) 2013 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

#ifndef HU_VARADIISTVAN_SCPL_POSIX_THREAD_H
#define HU_VARADIISTVAN_SCPL_POSIX_THREAD_H
//------------------------------------------------------------------------------

#include <pthread.h>
#include <unistd.h>

//------------------------------------------------------------------------------

namespace hu { namespace varadiistvan { namespace scpl {

//------------------------------------------------------------------------------

/**
 * A pthreads-based thread implementation.
 */
class Thread
{
public:
    /**
     * Sleep for the given number of milliseconds.
     */
    static void sleep(unsigned millis);

private:
    /**
     * The thread-runner function.
     */
    static void* threadFn(void* arg);

    /**
     * Indicate if the thread is detached or not.
     */
    bool detached;

    /**
     * The posix thread handle.
     */
    pthread_t thread;

public:
    /**
     * Construct the thread
     */
    Thread(bool detached = false);

    /**
     * Virtual destructor.
     */
    virtual ~Thread();

    /**
     * Start the thread.
     *
     * @return if the thread could be started
     */
    bool start();

    /**
     * Perform the thread's real operation.
     */
    virtual void run() = 0;

    /**
     * Wait for the thread to quit.
     */
    void join();
};

//------------------------------------------------------------------------------
// Inline definitions
//------------------------------------------------------------------------------

inline void Thread::sleep(unsigned millis)
{
    usleep(static_cast<useconds_t>(millis)*1000);
}

//------------------------------------------------------------------------------

inline Thread::Thread(bool detached) :
    detached(detached)
{
}

//------------------------------------------------------------------------------

inline Thread::~Thread()
{
}

//------------------------------------------------------------------------------

inline void Thread::join()
{
    pthread_join(thread, 0);
}

//------------------------------------------------------------------------------

} /* namespace hu::varadiistvan::scpl */ } /* namespace hu::varadiistvan */ } /* namespace hu */

//------------------------------------------------------------------------------
#endif // HU_VARADIISTVAN_SCPL_POSIX_THREAD_H

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
