// Copyright (c) 2013 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

//------------------------------------------------------------------------------

#include "BufferedStream.h"

#include "ReadingBuffer.h"
#include "WritingBuffer.h"

//------------------------------------------------------------------------------

using hu::varadiistvan::scpl::io::BufferedStream;

//------------------------------------------------------------------------------

BufferedStream::BufferedStream(Waiter* waiter, HANDLE handle,
                               size_t readingCapacity, size_t writingCapacity) :
    Overlappable(waiter, handle),
    readingBuffer( (readingCapacity==0) ? 0 :
                   new ReadingBuffer(readingCapacity, this)),
    writingBuffer( (writingCapacity==0) ? 0 :
                   new WritingBuffer(writingCapacity, this))
{
}

//------------------------------------------------------------------------------

BufferedStream::~BufferedStream()
{
    delete writingBuffer;
    delete readingBuffer;
}

//------------------------------------------------------------------------------

ssize_t BufferedStream::read(void* dest, size_t length, Overlapped* overlapped)
{
    DWORD numRead = 0;
    if (overlapped!=0) overlapped->reset();
    bool result = ReadFile(handle, dest, length,
                           (overlapped==0) ? &numRead : 0,
                           (overlapped==0) ? 0 : overlapped->get());
    DWORD error = GetLastError();
    if (!result &&
        (overlapped==0 ||
         (error!=ERROR_IO_PENDING && error!=ERROR_IO_INCOMPLETE)))
    {
        setErrorCode(error);
        return -1;
    } else if (overlapped==0 || overlapped->getResult(numRead, handle)) {
        return numRead;
    } else {
        return -1;
    }
}

//------------------------------------------------------------------------------

bool BufferedStream::write(const void* src, size_t length,
                           Overlapped* overlapped)
{
    if (overlapped!=0) overlapped->reset();
    bool result = WriteFile(handle, src, length, 0,
                            (overlapped==0) ? 0 : overlapped->get());
    DWORD error = GetLastError();
    if (!result &&
        (overlapped==0 ||
         (error!=ERROR_IO_PENDING && error!=ERROR_IO_INCOMPLETE)))
    {
        setErrorCode(error);
        return false;
    } else if (overlapped==0) {
        return result;
    } else {
        DWORD numWritten = 0;
        return overlapped->getResult(numWritten, handle);
    }
}

//------------------------------------------------------------------------------

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
